from sklearn.datasets import make_hastie_10_2
from sklearn.model_selection import train_test_split

import numpy as np

def createSomeData():

    n_samples = 100000
    test_size = 0.2

    # Hastie_10_2
    # X_i ~ Gaussian
    # sum of X_i^2 > Chi-squire(10, 0.5) 9.34, then 1, otherwise -1
    X, y_org = make_hastie_10_2(n_samples=n_samples) 
    z = np.random.randn(n_samples)
    y = y_org * z
    y[y > 0] = 1
    y[y <= 0] = 0
    X = np.hstack((X, z.reshape(n_samples,1)))
    n, m = X.shape
    X_train, X_test, y_train, y_test = train_test_split(X, y, test_size=test_size)

    return X_train, X_test, y_train, y_test

                                            


from whitebeam.base.regr import RegTree
from whitebeam.base.xgb import XGBTree

from sklearn.datasets import make_friedman1
from sklearn.datasets import make_friedman2
from sklearn.datasets import make_friedman3

from sklearn.model_selection import train_test_split
from sklearn.tree import DecisionTreeRegressor
from sklearn.preprocessing import PolynomialFeatures

import numpy as np
import time
from collections import Counter

import pytest

def test_regression_parallelisation(): 

    n_samples = 10000
    max_depth = 5
    test_size = 0.2
    X, y = make_friedman1(n_samples=n_samples) 
    poly = PolynomialFeatures(degree=3)
    X = poly.fit_transform(X)

    X_train, X_test, y_train, y_test = train_test_split(X, y, 
                                        test_size=test_size)

    model_p1 = RegTree(max_depth=max_depth, n_jobs=1)
    model_p2 = RegTree(max_depth=max_depth, n_jobs=2)

    start = time.time()
    model_p1.fit(X_train, y_train)
    time_fit_p1 = time.time() - start

    start = time.time()
    model_p2.fit(X_train, y_train)
    time_fit_p2 = time.time() - start

    assert time_fit_p2 < 5

def test_xgboost_parallelisation(): 

    n_samples = 10000
    max_depth = 5
    test_size = 0.2
    X, y = make_friedman1(n_samples=n_samples) 
    poly = PolynomialFeatures(degree=3)
    X = poly.fit_transform(X)

    X_train, X_test, y_train, y_test = train_test_split(X, y, 
                                        test_size=test_size)

    model_p1 = XGBTree(max_depth=max_depth, n_jobs=1)
    model_p2 = XGBTree(max_depth=max_depth, n_jobs=2)

    start = time.time()
    model_p1.fit(X_train, y_train)
    time_fit_p1 = time.time() - start

    start = time.time()
    model_p2.fit(X_train, y_train)
    time_fit_p2 = time.time() - start

    assert time_fit_p2 < 5








from whitebeam.base.alpha import AlphaTree
from whitebeam.base.c45 import C45Tree
from whitebeam.base.gini import GiniTree
from whitebeam.base.regr import RegTree
from whitebeam.base.xgb import XGBTree

from sklearn.metrics import roc_auc_score
import numpy as np
import pytest

from test.utils import createSomeData

def test_c45_auc():

    X_train, X_test, y_train, y_test = createSomeData()

    max_depth = 4
    model = C45Tree(max_depth=max_depth)
    model.fit(X_train, y_train)
    y_hat = model.predict(X_test)
    auc = roc_auc_score(y_test, y_hat)
    
    assert auc > 0.5

def test_gini_auc():

    X_train, X_test, y_train, y_test = createSomeData()

    max_depth = 4
    model = GiniTree(max_depth=max_depth)
    model.fit(X_train, y_train)
    y_hat = model.predict(X_test)
    auc = roc_auc_score(y_test, y_hat)

    assert auc > 0.5
    
def test_alpha_auc():

    X_train, X_test, y_train, y_test = createSomeData()

    max_depth = 4
    model = AlphaTree(alpha=3.0, max_depth=max_depth)
    model.fit(X_train, y_train)
    y_hat = model.predict(X_test)
    auc = roc_auc_score(y_test, y_hat)

    assert auc > 0.5

def test_regression_auc():

    X_train, X_test, y_train, y_test = createSomeData()

    max_depth = 4
    model = RegTree(max_depth=max_depth)
    model.fit(X_train, y_train)
    y_hat = model.predict(X_test)
    auc = roc_auc_score(y_test, y_hat)

    assert auc > 0.5

def test_xgboost_auc():

    X_train, X_test, y_train, y_test = createSomeData()

    max_depth = 4
    model = XGBTree(max_depth=max_depth)
    model.fit(X_train, y_train)
    y_hat = model.predict(X_test)
    auc = roc_auc_score(y_test, y_hat)

    assert auc > 0.5




    





# setup for cython functions
# Whitebeam | Kieran Molloy | Lancaster University 2020

import argparse

from setuptools import setup, Extension, find_packages
from Cython.Distutils import build_ext
import numpy as np
import pathlib

# parser = argparse.ArgumentParser()
# group = parser.add_mutually_exclusive_group(required=True)

# group.add_argument('-c', '--cython', action='store_true')
# group.add_argument('-b', '--build', action='store_true')
# args = parser.parse_args()

NAME = "whitebeam"
VERSION = "0.0.1"
DESCR = "Whitebeam is a framework for creating decision tree functions."
REQUIRES = ["numpy", "cython", "joblib"]

AUTHOR = "Kieran Molloy"
EMAIL = "k.molloy@lancaster.ac.uk"

SRC_DIR = "whitebeam"
PACKAGES = [SRC_DIR]

ext_1 = Extension(SRC_DIR + ".core._whitebeam",
                  sources=[SRC_DIR + "/core/_whitebeam.pyx"],
                  libraries=[],
                  include_dirs=[np.get_include()])

# The directory containing this file
HERE = pathlib.Path(__file__).parent

# The text of the README file
README = (HERE / "README.md").read_text()

# This call to setup() does all the work

setup(
    name="Whitebeam",
    version="1.0.0",
    description="Whitebeam is a framework for creating decision tree functions.",
    long_description=README,
    long_description_content_type="text/markdown",
    url="https://github.com/K-Molloy/whitebeam",
    author="Kieran Molloy",
    author_email="k.molloy@lancaster.ac.uk",
    license="MIT",
    classifiers=[
        "License :: OSI Approved :: MIT License",
        "Programming Language :: Python :: 3",
        "Programming Language :: Python :: 3.7",
    ],
    packages = find_packages()
    #ext_modules = [ext_1],
    #cmdclass={"build_ext": build_ext},
    #zip_safe=False
)